######### HIV biomarker ##########
library(rms)
library(mlt)
library(tram)
library(multcomp)
library(parallel)
library(tidyverse)
library(magrittr)
library(Hmisc)
library(kableExtra)
library(rlist)
library(latex2exp)

# load data
dat <- read.csv('data/koethe-biomarker-data.csv', stringsAsFactors = FALSE)

# function
source('data/sim_function.R')

# convert to factors for describe
dat.factor <- dat 
dat.factor[, c(3,4,6,7)] <- apply(dat.factor[, c(3,4,6,7)], 2, as.factor) %>% 
  as.data.frame()


#####################
####### hscrp #######
#####################

cpm.probit <- orm(hscrp ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                  data=dat, x=TRUE, y=TRUE, family=probit)

cpm.probit <- orm(hscrp ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                 data=dat, x=TRUE, y=TRUE)

mlt.probit <- as.mlt(BoxCox(hscrp ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                            data=dat, order = 10, 
                            support = quantile(dat$hscrp, prob = c(.05, .95)),
                            add = c(-10, 20),
                            extrapolate = TRUE))

mlt.probit <- as.mlt(Colr(hscrp ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                         data=dat, order = 10, 
                         support = quantile(dat$hscrp, prob = c(.05, .95)),
                         add = c(-10, 20),
                         extrapolate = TRUE))


### alpha(y) transformation ###
## MLT
cutpoints <- seq(quantile(dat$hscrp, prob = .05), quantile(dat$hscrp, prob = .95), by=0.01)
basis <- mlt.probit$model$bases[[1]]
n.v <- mlt.probit$df - length(mlt.probit$shiftcoef)  #M+1
v <- coef(mlt.probit)[1:n.v]
ay <- basis(cutpoints)
alpha.probit.mlt <- ay %*% v

## CPM
alpha.probit.cpm <- -cpm.probit$coefficients[1:(length(cpm.probit$yunique)-1)]

# generate new hscrp and take cubic root
new.bmi <- seq(20,45, by=0.1)
# (+3+1 means splines and dependent variables)
new.data <- matrix(NA, ncol=dim(cpm.probit$x)[2]+1, nrow=length(new.bmi)) %>% 
  as.data.frame() 
colnames(new.data) <- c("hscrp", colnames(mlt.probit$data)[2:8])

# most frequent level or median for other covariates
new.data$study.linc <- 1
new.data$male <- 1
new.data$age <- median(dat$age)
new.data$nonwhite <- 0
new.data$smoker <- 0
new.data$cd4 <- median(dat$cd4)
new.data$bmi <- new.bmi

# combine new data with the original training data
dat.mlt <- rbind(dat[, colnames(new.data)], new.data)


### mlt model for predicting change in hscrp associated with bmi
mlt.probit.var <- as.mlt(BoxCox(hscrp ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                                data=dat.mlt, order = 10, 
                                support = quantile(dat$hscrp, prob = c(.05, .95)),
                                add = c(-10, 20),
                                extrapolate = TRUE))

### Mean ###
# CPM 
est.mean.orm.probit <- mean.orm(cpm.probit,new.data = new.data[,2:8], se=TRUE)

# MLT
mlt.probit.mean <- as.mlt(BoxCox(hscrp ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                                 data=dat.mlt, order = 10, 
                                 support = range(dat$hscrp),
                                 bounds = c(-Inf, Inf),
                                 extrapolate = TRUE))

est.mean.mlt.probit <- mean.mlt(mlt_y=mlt.probit.var, mlt_y_mean=mlt.probit.mean,
                                data=dat.mlt, newdata = new.data[,2:8])


# to store var (for comparison with log(hsCRP))
est.mean.orm.probit1 <- est.mean.orm.probit
est.mean.orm.probit1 <- est.mean.orm.probit
est.mean.mlt.probit1 <- est.mean.mlt.probit
est.mean.mlt.probit1 <- est.mean.mlt.probit


### Median ###
## CPM
est.median.orm.probit <- quantile.orm(cpm.probit,new.data = new.data[,2:8], probs=0.5, se=TRUE)

## MLT (slow)
# MLT - probit
est.median.mlt.probit <- quantile.mlt(mlt.probit.var, new.data[,2:8], prob=0.5)
est.median.mlt.probit <- list.rbind(est.median.mlt.probit)


### PI ###
## CPM
pi.cpm <- pnorm(10*coef(cpm.probit)["bmi"], sd=sqrt(2))
## MLT
pi.mlt <- pnorm(10*coef(mlt.probit)["bmi"], sd=sqrt(2))

## confidence interval
cpm.probit.bmi.se <- (cpm.probit$var %>% diag)["bmi"] %>% sqrt
(coef(cpm.probit)["bmi"]*10 + c(-1,1)*qnorm(0.975)*cpm.probit.bmi.se*10) %>% pnorm(mean=0,sd=sqrt(2))

mlt.probit.bmi.se <- (vcov(mlt.probit) %>% diag)["bmi"] %>% sqrt
(10*coef(mlt.probit)["bmi"] + c(-1,1)*qnorm(0.975)*mlt.probit.bmi.se*10) %>% pnorm(mean=0,sd=sqrt(2))


### plots ###
par(mfrow=c(2,2))
hist(dat$hscrp, breaks=length(unique(dat$hscrp)), 
     xlab="hsCRP (mg/l)", main="(A) Histogram of hsCRP")

plot(cpm.probit$yunique[6:142], c(alpha.probit.cpm, Inf)[6:142], 
     type="s", ylab=expression(paste(hat(alpha), "(y)")), xlab="hsCRP (mg/l)", main="(B) Transformation",
     ylim=c(0,5))
lines(cutpoints, alpha.probit.mlt, col=2)
legend("top", c("CPM (probit link)", "MLT (probit link)" ), 
       col=c(1,2), lty=c(1,1),bty="n", cex=0.8)

plot(new.bmi, est.mean.orm.probit[,1], cex= 0.01, ylim=c(1,10), ylab="hsCRP (mg/l)", xlab=TeX("BMI (kg/m$^2$)"), type='n', main="(C) Mean")
polygon(c(new.bmi, rev(new.bmi)), c(est.mean.orm.probit[,3], rev(est.mean.orm.probit[,4])), col = "grey", border = NA)
polygon(c(new.bmi, rev(new.bmi)), c(est.mean.mlt.probit[,3], rev(est.mean.mlt.probit[,4])), col =colors()[560], border = NA)
lines(new.bmi, est.mean.orm.probit[,1], cex=0.01)
lines(new.bmi, est.mean.orm.probit[,3], cex=0.01, lty=2)
lines(new.bmi, est.mean.orm.probit[,4], cex=0.01, lty=2)
lines(new.bmi, est.mean.mlt.probit[,1], cex=0.01, col=2)
lines(new.bmi, est.mean.mlt.probit[,3], cex=0.01, lty=2, col=2)
lines(new.bmi, est.mean.mlt.probit[,4], cex=0.01, lty=2, col=2)
legend("top", c("CPM (probit link)", "MLT (probit link)" ), 
       col=c(1,2), lty=c(1,1),bty="n", cex=0.8)

plot(new.bmi, est.median.orm.probit[[1]], cex= 0.01, ylim=c(0,8), ylab="hsCRP (mg/l)", xlab=TeX("BMI (kg/m$^2$)"), type='n', main="(D) Median")
polygon(c(new.bmi, rev(new.bmi)), c(est.median.orm.probit$lb, rev(est.median.orm.probit$ub)), col = "grey", border = NA)
polygon(c(new.bmi, rev(new.bmi)), c(est.median.mlt.probit[,2], rev(est.median.mlt.probit[,3])), col =colors()[560], border = NA)
lines(new.bmi, est.median.orm.probit[[1]], cex=0.01)
lines(new.bmi, est.median.orm.probit[[2]], cex=0.01, lty=2)
lines(new.bmi, est.median.orm.probit[[3]], cex=0.01, lty=2)
lines(new.bmi, est.median.mlt.probit[,1], cex=0.01, col=2)
lines(new.bmi, est.median.mlt.probit[,2], cex=0.01, lty=2, col=2)
lines(new.bmi, est.median.mlt.probit[,3], cex=0.01, lty=2, col=2)
legend("top", c("CPM (probit link)", "MLT (probit link)" ), 
       col=c(1,2), lty=c(1,1), bty="n", cex=0.8)

#################### 
#### log(hsCRP) ####
#################### 
dat$log_hscrp <- log(dat$hscrp)

cpm.probit <- orm(log_hscrp ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                  data=dat, x=TRUE, y=TRUE, family=probit)

mlt.probit <- as.mlt(BoxCox(log_hscrp ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                            data=dat, order = 10, 
                            support = quantile(dat$log_hscrp, prob = c(.05, .95)),
                            add = c(-10, 10),
                            extrapolate = TRUE))


### alpha(y) ###
## MLT
cutpoints <- seq(quantile(dat$log_hscrp, prob = .05), quantile(dat$log_hscrp, prob = .95), by=0.01)
basis <- mlt.probit$model$bases[[1]]
n.v <- mlt.probit$df - length(mlt.probit$shiftcoef)  #M+1
v <- coef(mlt.probit)[1:n.v]
ay <- basis(cutpoints)
alpha.probit.mlt <- ay %*% v
## CPM
alpha.probit.cpm <- -cpm.probit$coefficients[1:(length(cpm.probit$yunique)-1)]


# generate new hscrp and take log
new.bmi <- seq(20,45, by=0.1)
# (+3+1 means splines and dependent variables)
new.data <- matrix(NA, ncol=dim(cpm.probit$x)[2]+1, nrow=length(new.bmi)) %>% as.data.frame() 
colnames(new.data) <- c("log_hscrp", colnames(mlt.probit$data)[2:8])

new.data$study.linc <- 1
new.data$male <- 1
new.data$age <- median(dat$age)
new.data$nonwhite <- 0
new.data$smoker <- 0
new.data$cd4 <- median(dat$cd4)
new.data$bmi <- new.bmi

# combine new data with the original training data
dat.mlt <- rbind(dat[, colnames(new.data)], new.data)

### mlt model for predicting change in log(hscrp) associated with bmi
mlt.probit.var <- as.mlt(BoxCox(log_hscrp ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                                data=dat.mlt, order = 10, 
                                support = quantile(dat$log_hscrp, prob = c(.05, .95)),
                                add = c(-2, 2),
                                extrapolate = TRUE))


### Mean ###
# CPM - probit
est.mean.orm.probit <- mean.orm(cpm.probit,new.data = new.data[,2:8], se=TRUE)
# MLT
## MLT - probit
mlt.probit.mean <- as.mlt(BoxCox(log_hscrp ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                                 data=dat.mlt, order = 10, 
                                 support = range(dat$log_hscrp),
                                 bounds = c(-Inf, Inf),
                                 extrapolate = TRUE))

est.mean.mlt.probit <- mean.mlt(mlt_y=mlt.probit.var, mlt_y_mean=mlt.probit.mean,
                                data=dat.mlt, newdata = new.data[,2:8])


### Median ###
## CPM
est.median.orm.probit <- quantile.orm(cpm.probit,new.data = new.data[,2:8], probs=0.5, se=TRUE)
## MLT (slow)
est.median.mlt.probit <- quantile.mlt(mlt.probit.var, new.data[,2:8], prob=0.5)
est.median.mlt.probit <- list.rbind(est.median.mlt.probit)


### PI ###
## CPM
pi.cpm <- pnorm(10*coef(cpm.probit)["bmi"], sd=sqrt(2))
## MLT
pi.mlt <- pnorm(10*coef(mlt.probit)["bmi"], sd=sqrt(2))
## confidence interval
cpm.probit.bmi.se <- (cpm.probit$var %>% diag)["bmi"] %>% sqrt
(coef(cpm.probit)["bmi"]*10 + c(-1,1)*qnorm(0.975)*cpm.probit.bmi.se*10) %>% pnorm(mean=0,sd=sqrt(2))

mlt.probit.bmi.se <- (vcov(mlt.probit) %>% diag)["bmi"] %>% sqrt
(10*coef(mlt.probit)["bmi"] + c(-1,1)*qnorm(0.975)*mlt.probit.bmi.se*10) %>% pnorm(mean=0,sd=sqrt(2))


#### Compare hsCRP and log(hsCRP) ####
# CPM - transform log scale back
mod1 <-  orm(hscrp ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
             data=dat, x=TRUE, y=TRUE, family=probit)
mod <-  orm(log_hscrp ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
            data=dat, x=TRUE, y=TRUE, family=probit)
order.y <- mod1$yunique
n.alpha <- length(order.y)-1
xb <- as.matrix(new.data[,2:8])%*%matrix(coef(mod)[colnames(new.data[,2:8])])
m.alpha <- mod$coef[1:n.alpha]
lb <- t(outer(m.alpha, xb, "+")[,,1])
m.s <- mod$trans$cumprob(lb)
m.f <- t(apply(m.s, 1, FUN=function(x) c(1,x[1:n.alpha]) - c(x[1:n.alpha], 0)))
c.mean <- apply(m.f, 1, FUN=function(x) sum(x*order.y)) # transformed back

## MLT- transformed log scale back
mlt_y <- as.mlt(BoxCox(log_hscrp ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                       data=dat.mlt, order = 10,
                       support = quantile(dat$log_hscrp, prob = c(.05, .95)),
                       add = c(-2, 2),
                       extrapolate = TRUE))

q <- mkgrid(mlt_y, n = 1001)[['log_hscrp']]
d <- predict(mlt_y, newdata = new.data[,2:8], type = "distribution", q = mkgrid(mlt_y, n = 1001)[['log_hscrp']])
m.f <- apply(d, 2, FUN=function(x) c(x[1:dim(d)[1]])-c(0,x[1:dim(d)[1]-1]))
m.mean <- apply(m.f, 2, function(x) x %*% exp(q))


### plots ###
par(mfrow=c(2,2))
hist(log(dat$hscrp), breaks=length(unique(dat$hscrp)), 
     xlab="log(hsCRP)", main="(A) Histogram of log(hsCRP)")

plot(cpm.probit$yunique[6:142], c(alpha.probit.cpm, Inf)[6:142], 
     type="s", ylab=expression(paste(hat(alpha), "(y)")), xlab="log(hsCRP)", main="(B) Transformation",
     ylim=c(0,4))
lines(cutpoints, alpha.probit.mlt, col=2)
legend("top", c("CPM (probit link)", "MLT (probit link)" ), 
       col=c(1,2), lty=c(1,1),bty="n", cex=0.8)

plot(new.bmi, est.mean.orm.probit[,1], cex= 0.01, ylim=c(-0.5,2), ylab="log(hsCRP)", xlab="BMI", type='n', main="(C) Mean")
polygon(c(new.bmi, rev(new.bmi)), c(est.mean.orm.probit[,3], rev(est.mean.orm.probit[,4])), col = "grey", border = NA)
polygon(c(new.bmi, rev(new.bmi)), c(est.mean.mlt.probit[,3], rev(est.mean.mlt.probit[,4])), col =colors()[560], border = NA)
lines(new.bmi, est.mean.orm.probit[,1], cex=0.01)
lines(new.bmi, est.mean.orm.probit[,3], cex=0.01, lty=2)
lines(new.bmi, est.mean.orm.probit[,4], cex=0.01, lty=2)
lines(new.bmi, est.mean.mlt.probit[,1], cex=0.01, col=2)
lines(new.bmi, est.mean.mlt.probit[,3], cex=0.01, lty=2, col=2)
lines(new.bmi, est.mean.mlt.probit[,4], cex=0.01, lty=2, col=2)
legend("top", c("CPM (probit link)", "MLT (probit link)" ), 
       col=c(1,2), lty=c(1,1),bty="n", cex=0.8)

plot(new.bmi, est.median.orm.probit[[1]], cex= 0.01, ylim=c(-0.5,2), ylab="log(hsCRP)", xlab="BMI", type='n', main="(D) Median")
polygon(c(new.bmi, rev(new.bmi)), c(est.median.orm.probit$lb, rev(est.median.orm.probit$ub)), col = "grey", border = NA)
polygon(c(new.bmi, rev(new.bmi)), c(est.median.mlt.probit[,2], rev(est.median.mlt.probit[,3])), col =colors()[560], border = NA)
lines(new.bmi, est.median.orm.probit[[1]], cex=0.01)
lines(new.bmi, est.median.orm.probit[[2]], cex=0.01, lty=2)
lines(new.bmi, est.median.orm.probit[[3]], cex=0.01, lty=2)
lines(new.bmi, est.median.mlt.probit[,1], cex=0.01, col=2)
lines(new.bmi, est.median.mlt.probit[,2], cex=0.01, lty=2, col=2)
lines(new.bmi, est.median.mlt.probit[,3], cex=0.01, lty=2, col=2)
legend("top", c("CPM (probit link)", "MLT (probit link)" ), 
       col=c(1,2), lty=c(1,1),bty="n", cex=0.8)

##################
###### sCD14 #####
##################
cpm.probit <- orm(cd14 ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                  data=dat, x=TRUE, y=TRUE, family=probit)

mlt.probit <- as.mlt(BoxCox(cd14 ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                            data=dat, order = 10, 
                            support = quantile(dat$cd14, prob = c(.05, .95)),
                            add = c(-10, 10),
                            extrapolate = TRUE))

### alpha(y) transformation ###
## MLT
cutpoints <- seq(quantile(dat$cd14, prob = .05), quantile(dat$cd14, prob = .95), by=0.01)
basis <- mlt.probit$model$bases[[1]]
n.v <- mlt.probit$df - length(mlt.probit$shiftcoef)  #M+1
v <- coef(mlt.probit)[1:n.v]
ay <- basis(cutpoints)
alpha.probit.mlt <- ay %*% v
## CPM
alpha.probit.cpm <- -cpm.probit$coefficients[1:(length(cpm.probit$yunique)-1)]

# generate new hscrp and take cubic root
new.bmi <- seq(20,45, by=0.1)
# (+3+1 means splines and dependent variables)
new.data <- matrix(NA, ncol=dim(cpm.probit$x)[2]+1, nrow=length(new.bmi)) %>% as.data.frame() 
colnames(new.data) <- c("cd14", colnames(mlt.probit$data)[2:8])

new.data$study.linc <- 1
new.data$male <- 1
new.data$age <- median(dat$age)
new.data$nonwhite <- 0
new.data$smoker <- 0
new.data$cd4 <- median(dat$cd4)
new.data$bmi <- new.bmi

# combine new data with the original training data
dat.mlt <- rbind(dat[, colnames(new.data)], new.data)

### mlt model for predicting change in sCD14 associated with bmi
mlt.probit.var <- as.mlt(BoxCox(cd14 ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                                data=dat.mlt, order = 10, 
                                support = quantile(dat$cd14, prob = c(.05, .95)),
                                add = c(-10, 10),
                                extrapolate = TRUE))


### Mean ###
# CPM - probit
est.mean.orm.probit <- mean.orm(cpm.probit,new.data = new.data[,2:8], se=TRUE)

# MLT
## MLT - probit
mlt.probit.mean <- as.mlt(BoxCox(cd14 ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                                 data=dat.mlt, order = 10, 
                                 support = range(dat$cd14),
                                 bounds = c(-Inf, Inf),
                                 extrapolate = TRUE))

est.mean.mlt.probit <- mean.mlt(mlt_y=mlt.probit.var, mlt_y_mean=mlt.probit.mean,
                                data=dat.mlt, newdata = new.data[,2:8])


### Median ###
## CPM
est.median.orm.probit <- quantile.orm(cpm.probit,new.data = new.data[,2:8], probs=0.5, se=TRUE)
## MLT 
est.median.mlt.probit <- quantile.mlt(mlt.probit.var, new.data[,2:8], prob=0.5)
est.median.mlt.probit <- list.rbind(est.median.mlt.probit)


### PI ###
## CPM
pi.cpm <- pnorm(10*coef(cpm.probit)["bmi"], sd=sqrt(2))
## MLT
pi.mlt <- pnorm(10*coef(mlt.probit)["bmi"], sd=sqrt(2))
## confidence interval
cpm.probit.bmi.se <- (cpm.probit$var %>% diag)["bmi"] %>% sqrt
(coef(cpm.probit)["bmi"]*10 + c(-1,1)*qnorm(0.975)*cpm.probit.bmi.se*10) %>% pnorm(mean=0,sd=sqrt(2))

mlt.probit.bmi.se <- (vcov(mlt.probit) %>% diag)["bmi"] %>% sqrt
(10*coef(mlt.probit)["bmi"] + c(-1,1)*qnorm(0.975)*mlt.probit.bmi.se*10) %>% pnorm(mean=0,sd=sqrt(2))


### plot ###
par(mfrow=c(2,2))

hist(dat$cd14, breaks=length(unique(dat$cd14)), 
     xlab=TeX("sCD14 ($\\mu$g/ml)"), main="(A) Histogram of sCD14")

plot(cpm.probit$yunique[11:158], c(alpha.probit.cpm, Inf)[11:158],
     type="s", ylab=expression(paste(hat(alpha), "(y)")), xlab=TeX("sCD14 ($\\mu$g/ml)"), main="(B) Transformation",
     ylim=c(-3,3))
lines(cutpoints, alpha.probit.mlt, col=2)
legend("top", c("CPM (probit link)", "MLT (probit link)" ), col=c(1,2), lty=c(1,1),bty="n")

plot(new.bmi, est.mean.orm.probit[,1], cex= 0.01, ylim=c(1,3), ylab=TeX("sCD14 ($\\mu$g/ml)"), xlab=TeX("BMI (kg/m$^2$)"), type='n', main="(C) Mean")
polygon(c(new.bmi, rev(new.bmi)), c(est.mean.orm.probit[,3], rev(est.mean.orm.probit[,4])), col = "grey", border = NA)
polygon(c(new.bmi, rev(new.bmi)), c(est.mean.mlt.probit[,3], rev(est.mean.mlt.probit[,4])), col =colors()[560], border = NA)
lines(new.bmi, est.mean.orm.probit[,1], cex=0.01)
lines(new.bmi, est.mean.orm.probit[,3], cex=0.01, lty=2)
lines(new.bmi, est.mean.orm.probit[,4], cex=0.01, lty=2)
lines(new.bmi, est.mean.mlt.probit[,1], cex=0.01, col=2)
lines(new.bmi, est.mean.mlt.probit[,3], cex=0.01, lty=2, col=2)
lines(new.bmi, est.mean.mlt.probit[,4], cex=0.01, lty=2, col=2)
legend("top", c("CPM (probit link)", "MLT (probit link)" ), col=c(1,2), lty=c(1,1),bty="n")

plot(new.bmi, est.median.orm.probit[[1]], cex= 0.01, ylim=c(1,3), ylab=TeX("sCD14 ($\\mu$g/ml)"), xlab=TeX("BMI (kg/m$^2$)"), type='n', main="(D) Median")
polygon(c(new.bmi, rev(new.bmi)), c(est.median.orm.probit$lb, rev(est.median.orm.probit$ub)), col = "grey", border = NA)
polygon(c(new.bmi, rev(new.bmi)), c(est.median.mlt.probit[,2], rev(est.median.mlt.probit[,3])), col =colors()[560], border = NA)
lines(new.bmi, est.median.orm.probit[[1]], cex=0.01)
lines(new.bmi, est.median.orm.probit[[2]], cex=0.01, lty=2)
lines(new.bmi, est.median.orm.probit[[3]], cex=0.01, lty=2)
lines(new.bmi, est.median.mlt.probit[,1], cex=0.01, col=2)
lines(new.bmi, est.median.mlt.probit[,2], cex=0.01, lty=2, col=2)
lines(new.bmi, est.median.mlt.probit[,3], cex=0.01, lty=2, col=2)
legend("top", c("CPM (probit link)", "MLT (probit link)" ), col=c(1,2), lty=c(1,1),bty="n")

########################
#### IL-1-beta ####
########################
cpm.probit <- orm(il_1_beta ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                  data=dat, x=TRUE, y=TRUE, family=probit)

# mlt survival
dat$il_1_beta_dl <- ifelse(dat$il_1_beta==0, 0, 1) #lower detection limit indicator
dat$il_1_beta_surv <- Surv(dat$il_1_beta, dat$il_1_beta_dl, type="left")

mlt.probit <- as.mlt(BoxCox(il_1_beta_surv ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                            data=dat, order = 6, 
                            support = quantile(dat$il_1_beta, prob = c(.05, .95)),
                            add = c(-10, 25),
                            extrapolate = TRUE))

### alpha(y) ###
## MLT
cutpoints <- seq(quantile(dat$il_1_beta, prob = .05), quantile(dat$il_1_beta, prob = .95), by=0.01)
basis <- mlt.probit$model$bases[[1]]
n.v <- mlt.probit$df - length(mlt.probit$shiftcoef)  #M+1
v <- coef(mlt.probit)[1:n.v]
ay <- basis(cutpoints)
alpha.probit.mlt <- ay %*% v
## CPM
alpha.probit.cpm <- -cpm.probit$coefficients[1:(length(cpm.probit$yunique)-1)]

# generate new hscrp and take cubic root
new.bmi <- seq(20,45, by=0.1)
# (+3+1 means splines and dependent variables)
new.data <- matrix(NA, ncol=dim(cpm.probit$x)[2]+1, nrow=length(new.bmi)) %>% as.data.frame() 
colnames(new.data) <- c("il_1_beta", colnames(mlt.probit$data)[2:8])

new.data$study.linc <- 1
new.data$male <- 1
new.data$age <- median(dat$age)
new.data$nonwhite <- 0
new.data$smoker <- 0
new.data$cd4 <- median(dat$cd4)
new.data$bmi <- new.bmi

# combine new data with the original training data
dat.mlt <- rbind(dat[, colnames(new.data)], new.data)
dat.mlt$il_1_beta_dl <-  ifelse(dat.mlt$il_1_beta==0, 0, 1) #lower detection limit indicator
dat.mlt$il_1_beta_surv <- Surv(dat.mlt$il_1_beta, dat.mlt$il_1_beta_dl, type="left")

### mlt model for predicting change in il-1-beta associated with bmi
mlt.probit.var <- as.mlt(BoxCox(il_1_beta_surv ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                                data=dat.mlt, order = 6, 
                                support = quantile(dat$il_1_beta, prob = c(.05, .95)),
                                add = c(-10, 25),
                                extrapolate = TRUE))

### Mean ###
# CPM - probit
est.mean.orm.probit <- mean.orm(cpm.probit,new.data = new.data[,2:8], se=TRUE)
# MLT
mlt.probit.mean <- as.mlt(BoxCox(il_1_beta_surv ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                                 data=dat.mlt, order = 6, 
                                 support = range(dat$il_1_beta),
                                 bounds = c(-Inf, Inf),
                                 extrapolate = TRUE))

est.mean.mlt.probit <- mean.mlt(mlt_y=mlt.probit.var, mlt_y_mean=mlt.probit.mean,
                                data=dat.mlt, newdata = new.data[,2:8])


### Median ###
## CPM
est.median.orm.probit <- quantile.orm(cpm.probit,new.data = new.data[,2:8], probs=0.5, se=TRUE)
## MLT
est.median.mlt.probit <- quantile.mlt(mlt.probit.var, new.data[,2:8], prob=0.5)
est.median.mlt.probit <- list.rbind(est.median.mlt.probit)


### PI ###
## CPM
pi.cpm <- pnorm(10*coef(cpm.probit)["bmi"], sd=sqrt(2))
## MLT
pi.mlt <- pnorm(10*coef(mlt.probit)["bmi"], sd=sqrt(2))

## confidence interval
cpm.probit.bmi.se <- (cpm.probit$var %>% diag)["bmi"] %>% sqrt
(coef(cpm.probit)["bmi"]*10 + c(-1,1)*qnorm(0.975)*cpm.probit.bmi.se*10) %>% pnorm(mean=0,sd=sqrt(2))

mlt.probit.bmi.se <- (vcov(mlt.probit) %>% diag)["bmi"] %>% sqrt
(10*coef(mlt.probit)["bmi"] + c(-1,1)*qnorm(0.975)*mlt.probit.bmi.se*10) %>% pnorm(mean=0,sd=sqrt(2))


#### plots ####
par(mfrow=c(2,2))
hist(dat$il_1_beta, breaks=length(unique(dat$il_1_beta)), 
     xlab=expression(paste("IL-6-",beta)), 
     main=expression(bold(paste("(A) Histogram of IL-6-",beta))))


plot(cpm.probit$yunique[1:104], c(alpha.probit.cpm, Inf)[1:104],
     type="s", 
     ylab=expression(paste(hat(alpha), "(y)")), xlab=TeX("IL-1-$\\beta$ (ng/ml)"), main="(B) Transformation",
     ylim=c(-0.5,2.5))
lines(cutpoints, alpha.probit.mlt, col=2)
legend("top", c("CPM (probit link)", "MLT (probit link)" ), 
       col=c(1,2), lty=c(1,1),bty="n", cex=0.8)

plot(new.bmi, est.mean.orm.probit[,1], cex= 0.01, ylim=c(0,1.5), ylab=TeX("IL-1-$\\beta$ (ng/ml)"), xlab=TeX("BMI (kg/m$^2$)"), type='n', main="(C) Mean")
polygon(c(new.bmi, rev(new.bmi)), c(est.mean.orm.probit[,3], rev(est.mean.orm.probit[,4])), col = "grey", border = NA)
polygon(c(new.bmi, rev(new.bmi)), c(est.mean.mlt.probit[,3], rev(est.mean.mlt.probit[,4])), col =colors()[560], border = NA)
lines(new.bmi, est.mean.orm.probit[,1], cex=0.01)
lines(new.bmi, est.mean.orm.probit[,3], cex=0.01, lty=2)
lines(new.bmi, est.mean.orm.probit[,4], cex=0.01, lty=2)
lines(new.bmi, est.mean.mlt.probit[,1], cex=0.01, col=2)
lines(new.bmi, est.mean.mlt.probit[,3], cex=0.01, lty=2, col=2)
lines(new.bmi, est.mean.mlt.probit[,4], cex=0.01, lty=2, col=2)
legend("top", c("CPM (probit link)", "MLT (probit link)" ), 
       col=c(1,2), lty=c(1,1),bty="n", cex=0.8)

plot(new.bmi, est.median.orm.probit[[1]], cex= 0.01, ylim=c(-0.5,1), ylab=TeX("IL-1-$\\beta$ (ng/ml)"), xlab=TeX("BMI (kg/m$^2$)"), type='n', main="(D) Median")
polygon(c(new.bmi, rev(new.bmi)), c(est.median.orm.probit$lb, rev(est.median.orm.probit$ub)), col = "grey", border = NA)
polygon(c(new.bmi, rev(new.bmi)), c(est.median.mlt.probit[,2], rev(est.median.mlt.probit[,3])), col =colors()[560], border = NA)
lines(new.bmi, est.median.orm.probit[[1]], cex=0.01)
lines(new.bmi, est.median.orm.probit[[2]], cex=0.01, lty=2)
lines(new.bmi, est.median.orm.probit[[3]], cex=0.01, lty=2)
lines(new.bmi, est.median.mlt.probit[,1], cex=0.01, col=2)
lines(new.bmi, est.median.mlt.probit[,2], cex=0.01, lty=2, col=2)
lines(new.bmi, est.median.mlt.probit[,3], cex=0.01, lty=2, col=2)
legend("top", c("CPM (probit link)", "MLT (probit link)" ), 
       col=c(1,2), lty=c(1,1),bty="n", cex=0.8)



########################
######### IL-6 #######
########################
cpm.probit <- orm(il_6 ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                  data=dat, x=TRUE, y=TRUE, family=probit)

mlt.probit <- as.mlt(BoxCox(il_6 ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                            data=dat, order = 10, 
                            support = quantile(dat$il_6, prob = c(.05, .95)),
                            add = c(-10, 20),
                            extrapolate = TRUE))


### alpha(y) ####
## MLT
cutpoints <- seq(quantile(dat$il_6, prob = .05), quantile(dat$il_6, prob = .95), by=0.01)
basis <- mlt.probit$model$bases[[1]]
n.v <- mlt.probit$df - length(mlt.probit$shiftcoef)  #M+1
v <- coef(mlt.probit)[1:n.v]
ay <- basis(cutpoints)
alpha.probit.mlt <- ay %*% v
## CPM
alpha.probit.cpm <- -cpm.probit$coefficients[1:(length(cpm.probit$yunique)-1)]


# generate new hscrp and take cubic root
new.bmi <- seq(20,45, by=0.1)
# (+3+1 means splines and dependent variables)
new.data <- matrix(NA, ncol=dim(cpm.probit$x)[2]+1, nrow=length(new.bmi)) %>% as.data.frame() 
colnames(new.data) <- c("il_6", colnames(mlt.probit$data)[2:8])

new.data$study.linc <- 1
new.data$male <- 1
new.data$age <- median(dat$age)
new.data$nonwhite <- 0
new.data$smoker <- 0
new.data$cd4 <- median(dat$cd4)
new.data$bmi <- new.bmi

# combine new data with the original training data
dat.mlt <- rbind(dat[, colnames(new.data)], new.data)


### mlt model for predicting change in IL-6 associated with bmi
mlt.probit.var <- as.mlt(BoxCox(il_6 ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                                data=dat.mlt, order = 10, 
                                support = quantile(dat$il_6, prob = c(.05, .95)),
                                add = c(-10, 20),
                                extrapolate = TRUE))

 
### Mean ###
# CPM 
est.mean.orm.probit <- mean.orm(cpm.probit,new.data = new.data[,2:8], se=TRUE)

# MLT
mlt.probit.mean <- as.mlt(BoxCox(il_6 ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                                 data=dat.mlt, order = 10, 
                                 support = range(dat$il_6),
                                 bounds = c(-Inf, Inf),
                                 extrapolate = TRUE))

est.mean.mlt.probit <- mean.mlt(mlt_y=mlt.probit.var, mlt_y_mean=mlt.probit.mean,
                                data=dat.mlt, newdata = new.data[,2:8])


### Median ###
## CPM
est.median.orm.probit <- quantile.orm(cpm.probit,new.data = new.data[,2:8], probs=0.5, se=TRUE)
## MLT 
est.median.mlt.probit <- quantile.mlt(mlt.probit.var, new.data[,2:8], prob=0.5)
est.median.mlt.probit <- list.rbind(est.median.mlt.probit)


### PI ###
## CPM
pi.cpm <- pnorm(10*coef(cpm.probit)["bmi"], sd=sqrt(2))
## MLT
pi.mlt <- pnorm(10*coef(mlt.probit)["bmi"], sd=sqrt(2))

## confidence interval
cpm.probit.bmi.se <- (cpm.probit$var %>% diag)["bmi"] %>% sqrt
(coef(cpm.probit)["bmi"]*10 + c(-1,1)*qnorm(0.975)*cpm.probit.bmi.se*10) %>% pnorm(mean=0,sd=sqrt(2))

mlt.probit.bmi.se <- (vcov(mlt.probit) %>% diag)["bmi"] %>% sqrt
(10*coef(mlt.probit)["bmi"] + c(-1,1)*qnorm(0.975)*mlt.probit.bmi.se*10) %>% pnorm(mean=0,sd=sqrt(2))


### plot ###
par(mfrow=c(2,2))

hist(dat$il_6, breaks=length(unique(dat$il_6)), 
     xlab="IL-6 (pg/ml)", main="(A) Histogram of IL-6")

plot(cpm.probit$yunique[5:164], c(alpha.probit.cpm, Inf)[5:164], 
     type="s", ylab=expression(paste(hat(alpha), "(y)")), 
     xlab="IL-6 (pg/ml)", main="(B) Transformation",
     ylim=c(-1,5))
lines(cutpoints, alpha.probit.mlt, col=2)
legend("top", c("CPM (probit link)", "MLT (probit link)" ), 
       col=c(1,2), lty=c(1,1),bty="n", cex=0.8)

plot(new.bmi, est.mean.orm.probit[,1], cex= 0.01, ylim=c(1,7), ylab="IL-6 (pg/ml)", xlab=TeX("BMI (kg/m$^2$)"), type='n', main="(C) Mean")
polygon(c(new.bmi, rev(new.bmi)), c(est.mean.orm.probit[,3], rev(est.mean.orm.probit[,4])), col = "grey", border = NA)
polygon(c(new.bmi, rev(new.bmi)), c(est.mean.mlt.probit[,3], rev(est.mean.mlt.probit[,4])), col =colors()[560], border = NA)
lines(new.bmi, est.mean.orm.probit[,1], cex=0.01)
lines(new.bmi, est.mean.orm.probit[,3], cex=0.01, lty=2)
lines(new.bmi, est.mean.orm.probit[,4], cex=0.01, lty=2)
lines(new.bmi, est.mean.mlt.probit[,1], cex=0.01, col=2)
lines(new.bmi, est.mean.mlt.probit[,3], cex=0.01, lty=2, col=2)
lines(new.bmi, est.mean.mlt.probit[,4], cex=0.01, lty=2, col=2)
legend("top", c("CPM (probit link)", "MLT (probit link)" ), 
       col=c(1,2), lty=c(1,1),bty="n", cex=0.8)

plot(new.bmi, est.median.orm.probit[[1]], cex= 0.01, ylim=c(1,6), ylab="IL-6 (pg/ml)", xlab=TeX("BMI (kg/m$^2$)"), type='n', main="(D) Median")
polygon(c(new.bmi, rev(new.bmi)), c(est.median.orm.probit$lb, rev(est.median.orm.probit$ub)), col = "grey", border = NA)
polygon(c(new.bmi, rev(new.bmi)), c(est.median.mlt.probit[,2], rev(est.median.mlt.probit[,3])), col =colors()[560], border = NA)
lines(new.bmi, est.median.orm.probit[[1]], cex=0.01)
lines(new.bmi, est.median.orm.probit[[2]], cex=0.01, lty=2)
lines(new.bmi, est.median.orm.probit[[3]], cex=0.01, lty=2)
lines(new.bmi, est.median.mlt.probit[,1], cex=0.01, col=2)
lines(new.bmi, est.median.mlt.probit[,2], cex=0.01, lty=2, col=2)
lines(new.bmi, est.median.mlt.probit[,3], cex=0.01, lty=2, col=2)
legend("top", c("CPM (probit link)", "MLT (probit link)" ),
       col=c(1,2), lty=c(1,1),bty="n", cex=0.8)

########################
###### leptin #########
########################
cpm.probit <- orm(leptin ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                  data=dat, x=TRUE, y=TRUE, family=probit)

mlt.probit <- as.mlt(BoxCox(leptin ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                            data=dat, order = 10, 
                            support = quantile(dat$leptin, prob = c(.05, .95)),
                            add = c(-10, 150),
                            extrapolate = TRUE))

### alpha(y) ####
## MLT
cutpoints <- seq(quantile(dat$leptin, prob = .05), quantile(dat$leptin, prob = .95), by=0.01)
basis <- mlt.probit$model$bases[[1]]
n.v <- mlt.probit$df - length(mlt.probit$shiftcoef)  #M+1
v <- coef(mlt.probit)[1:n.v]
ay <- basis(cutpoints)
alpha.probit.mlt <- ay %*% v

## CPM
alpha.probit.cpm <- -cpm.probit$coefficients[1:(length(cpm.probit$yunique)-1)]


# generate new hscrp and take cubic root
new.bmi <- seq(20,45, by=0.1)
# (+3+1 means splines and dependent variables)
new.data <- matrix(NA, ncol=dim(cpm.probit$x)[2]+1, nrow=length(new.bmi)) %>% as.data.frame() 
colnames(new.data) <- c("leptin", colnames(mlt.probit$data)[2:8])

new.data$study.linc <- 1
new.data$male <- 1
new.data$age <- median(dat$age)
new.data$nonwhite <- 0
new.data$smoker <- 0
new.data$cd4 <- median(dat$cd4)
new.data$bmi <- new.bmi

# combine new data with the original training data
dat.mlt <- rbind(dat[, colnames(new.data)], new.data)

### mlt model for predicting change in leptin associated with bmi
mlt.probit.var <- as.mlt(BoxCox(leptin ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                                data=dat.mlt, order = 10, 
                                support = quantile(dat$leptin, prob = c(.05, .95)),
                                add = c(-10, 10),
                                extrapolate = TRUE))


### Mean ###
# CPM
est.mean.orm.probit <- mean.orm(cpm.probit,new.data = new.data[,2:8], se=TRUE)

# MLT
mlt.probit.mean <- as.mlt(BoxCox(leptin ~ study.linc + male + age + nonwhite + smoker + cd4 + bmi,
                                 data=dat.mlt, order = 10, 
                                 support = range(dat$leptin),
                                 bounds = c(-Inf, Inf),
                                 extrapolate = TRUE))

est.mean.mlt.probit <- mean.mlt(mlt_y=mlt.probit.var, mlt_y_mean=mlt.probit.mean,
                                data=dat.mlt, newdata = new.data[,2:8])


### Median ###
## CPM
est.median.orm.probit <- quantile.orm(cpm.probit,new.data = new.data[,2:8], probs=0.5, se=TRUE)

## MLT
est.median.mlt.probit <- quantile.mlt(mlt.probit.var, new.data[,2:8], prob=0.5)
est.median.mlt.probit <- list.rbind(est.median.mlt.probit)


### PI ###
## CPM
pi.cpm <- pnorm(10*coef(cpm.probit)["bmi"], sd=sqrt(2))
## MLT
pi.mlt <- pnorm(10*coef(mlt.probit)["bmi"], sd=sqrt(2))

## confidence interval
cpm.probit.bmi.se <- (cpm.probit$var %>% diag)["bmi"] %>% sqrt
(coef(cpm.probit)["bmi"]*10 + c(-1,1)*qnorm(0.975)*cpm.probit.bmi.se*10) %>% pnorm(mean=0,sd=sqrt(2))

mlt.probit.bmi.se <- (vcov(mlt.probit) %>% diag)["bmi"] %>% sqrt
(10*coef(mlt.probit)["bmi"] + c(-1,1)*qnorm(0.975)*mlt.probit.bmi.se*10) %>% pnorm(mean=0,sd=sqrt(2))


### plots ###
par(mfrow=c(2,2))
hist(dat$leptin, breaks=length(unique(dat$leptin)), 
     xlab="leptin (ng/ml)", main="(A) Histogram of leptin")


plot(cpm.probit$yunique[11:201], c(alpha.probit.cpm, Inf)[11:201],
     type="s", ylab=expression(paste(hat(alpha), "(y)")), xlab="leptin (ng/ml)", main="(B) Transformation",
     ylim=c(-1,7))
lines(cutpoints, alpha.probit.mlt, col=2)
legend("top", c("CPM (probit link)", "MLT (probit link)" ), 
       col=c(1,2), lty=c(1,1),bty="n", cex=0.8)

plot(new.bmi, est.mean.orm.probit[,1], cex= 0.01, ylim=c(0,30), ylab="leptin (ng/ml)", xlab=TeX("BMI (kg/m$^2$)"), type='n', main="(C) Mean")
polygon(c(new.bmi, rev(new.bmi)), c(est.mean.orm.probit[,3], rev(est.mean.orm.probit[,4])), col = "grey", border = NA)
polygon(c(new.bmi, rev(new.bmi)), c(est.mean.mlt.probit[,3], rev(est.mean.mlt.probit[,4])), col =colors()[560], border = NA)
lines(new.bmi, est.mean.orm.probit[,1], cex=0.01)
lines(new.bmi, est.mean.orm.probit[,3], cex=0.01, lty=2)
lines(new.bmi, est.mean.orm.probit[,4], cex=0.01, lty=2)
lines(new.bmi, est.mean.mlt.probit[,1], cex=0.01, col=2)
lines(new.bmi, est.mean.mlt.probit[,3], cex=0.01, lty=2, col=2)
lines(new.bmi, est.mean.mlt.probit[,4], cex=0.01, lty=2, col=2)
legend("top", c("CPM (probit link)", "MLT (probit link)" ), 
       col=c(1,2), lty=c(1,1),bty="n", cex=0.8)

plot(new.bmi, est.median.orm.probit[[1]], cex= 0.01, ylim=c(0,30), ylab="leptin (ng/ml)", xlab=TeX("BMI (kg/m$^2$)"), type='n', main="(D) Median")
polygon(c(new.bmi, rev(new.bmi)), c(est.median.orm.probit$lb, rev(est.median.orm.probit$ub)), col = "grey", border = NA)
polygon(c(new.bmi, rev(new.bmi)), c(est.median.mlt.probit[,2], rev(est.median.mlt.probit[,3])), col =colors()[560], border = NA)
lines(new.bmi, est.median.orm.probit[[1]], cex=0.01)
lines(new.bmi, est.median.orm.probit[[2]], cex=0.01, lty=2)
lines(new.bmi, est.median.orm.probit[[3]], cex=0.01, lty=2)
lines(new.bmi, est.median.mlt.probit[,1], cex=0.01, col=2)
lines(new.bmi, est.median.mlt.probit[,2], cex=0.01, lty=2, col=2)
lines(new.bmi, est.median.mlt.probit[,3], cex=0.01, lty=2, col=2)
legend("top", c("CPM (probit link)", "MLT (probit link)" ), 
       col=c(1,2), lty=c(1,1),bty="n", cex=0.8)

